@@ -4,6 +4,7 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gst/gst.h>
+#include <clutter-gtk/clutter-gtk.h>
 #include "cheese-avatar-chooser.h"
 
 static void
@@ -34,7 +35,7 @@ main (int argc, char **argv)
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
 
-  gtk_init (&argc, &argv);
+  gtk_clutter_init (&argc, &argv);
 
   window = cheese_avatar_chooser_new ();
   g_signal_connect (G_OBJECT (window), "response",
